# Tamito KAJIYAMA <2 April 2002>
# $Id: __init__.py,v 1.3 2003/11/29 23:31:23 kajiyama Exp $

import encodings.aliases

encodings.aliases.aliases.update({
    'euc_jp':                 'japanese.euc_jp',
    'eucjp':                  'japanese.euc_jp',
    'ujis':                   'japanese.euc_jp',
    'shift_jis':              'japanese.shift_jis',
    'sjis':                   'japanese.shift_jis',
    'ms932':                  'japanese.ms932',
    'cp932':                  'japanese.ms932',
    'windows_932':            'japanese.ms932',
    'windows_31j':            'japanese.ms932',
    'iso_2022_jp':            'japanese.iso_2022_jp',
    'jis_7':                  'japanese.iso_2022_jp',
    'iso_2022_jp_1':          'japanese.iso_2022_jp_1',
    'iso_2022_jp_ext':        'japanese.iso_2022_jp_ext',
    'jis_x_0201_katakana':    'japanese.jis_x_0201_katakana',
    'jis_x_0201_roman':       'japanese.jis_x_0201_roman',
    })
